#!/usr/bin/env wish

# This method parses the G-Fit result-file specified infile
# and calculates the MAP-description from it. Afterwards 
# the MAP is written into the standard XML-format and saved 
# at the location specified by outfile.
proc gfit_result_to_map {infile outfile} {
	set in_ph [open $infile]
	set branches 0
	set phases [list]
	set lambda [list]
	set init_prob [list]
	# read PH description
	while {![eof $in_ph]} {
        set line [gets $in_ph]
		if {$line == "# number of different Erlang branches, m"} {
			set branches [gets $in_ph]
		} elseif {$line == "# number of phases of i-th Erlang branch, i = 1,...,m"} {
			for {set i 0} {$i < $branches} {incr i} {
				lappend phases [gets $in_ph]
			}
		} elseif {$line == "# lambda values of i-th Erlang branch, i = 1,...,m"} {
			for {set i 0} {$i < $branches} {incr i} {
				lappend lambda [gets $in_ph]
			}
		} elseif {$line == "# initial probabilities of i-th Erlang branch, i = 1,...,m"} {
			for {set i 0} {$i < $branches} {incr i} {
				lappend init_prob [gets $in_ph]
			}
		}
	}
	# create MAP
	set d0 ""
	set d1 ""
	set pi ""
	set order 0
	# determine overall number of states
	for {set i 0} {$i < [llength $phases]} {incr i} {
		set order [expr $order + [lindex $phases $i]]
	}
	set prefix ""
	set postfix_length $order
	# for each branch of the PH distribution the corresponding part of the MAP is created
	for {set i 0} {$i < $branches} {incr i} {
		# convert first n - 1 phases of the branch
		for {set j 0} {$j < [lindex $phases $i] - 1} {incr j} {
			set d0line "$prefix -[lindex $lambda $i] [lindex $lambda $i] [string repeat "0 " [expr $postfix_length - 2]]"
			set d1line "[string repeat "0 " $order]"
			if {$d0 == ""} {
				set d0 "$d0line"
				set d1 "$d1line"
			} else {
				set d0 "$d0 \n$d0line"
				set d1 "$d1 \n$d1line"
			}
			set prefix "0 $prefix"
			set postfix_length [expr $postfix_length - 1]
		}
		# convert last phase of branch
		set d0line "$prefix -[lindex $lambda $i] [string repeat "0 " [expr $postfix_length - 1]]"
		set d1line ""
		for {set j 0} {$j < $branches} {incr j} {
			set d1line "$d1line [expr [lindex $lambda $i] * [lindex $init_prob $j]] [string repeat "0 " [expr [lindex $phases $j] - 1]]"
		}
		set prefix "0 $prefix"
		set postfix_length [expr $postfix_length - 1]
		if {$d0 == ""} {
			set d0 "$d0line"
			set d1 "$d1line"
		} else {
			set d0 "$d0 \n$d0line"
			set d1 "$d1 \n$d1line"
		}
	}
	for {set i 0} {$i < $branches} {incr i} {
		set pi "$pi [lindex $init_prob $i] [string repeat "0 " [expr [lindex $phases $i] - 1]]"
	}
	# write MAP file
	if {$outfile != ""} {
		set out_map [open $outfile w+]
		puts $out_map "<?xml version=\"1.0\"?>"
		puts $out_map "<profido>"
		puts $out_map "<dist>"
		puts $out_map "<hypererlangdist>"
		puts $out_map "<prob> $init_prob </prob>"
		puts $out_map "<phases> $phases </phases>"
		puts $out_map "<rates> $lambda </rates>"
		puts $out_map "</hypererlangdist>"
		puts $out_map "</dist>"
		puts $out_map "<ph>"
		puts $out_map "<states>$order</states>"
		puts $out_map "<pi> $pi </pi>"
		puts $out_map "<d0>$d0"
		puts $out_map "</d0>"
		puts $out_map "</ph>"
		puts $out_map "<map>"
		puts $out_map "<states>$order</states>"
		puts $out_map "<d0>$d0"
		puts $out_map "</d0>"
		puts $out_map "<d1>$d1"
		puts $out_map "</d1>"
		puts $out_map "</map>"
		puts $out_map "</profido>"
		close $out_map
	}
	#puts "d0 = $d0 \n\n"
	#puts "d1 = $d1 \n\n"
	close $in_ph
}


##################################################################################
##################################################################################
##################################################################################

# This script parses the G-Fit result-file specified by the first parameter
# and calculates the MAP-description from it. Afterwards the MAP is written 
# into the standard XML-format and saved at the location specified by the 
# second parameter. 
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
gfit_result_to_map $infile $outfile
exit

