#!/usr/bin/env wish

# helper function: get next line from MAP file, ignores comments and empty lines
proc result_nextline {infile} {
        set line ""
        while {($line == "") || ([string index $line 0] == "#")} {
                set line [string trim [gets $infile]]
        }
        return $line
}

# This method parses the EM result-file specified infile
# and calculates the MAP-description from it. Afterwards
# the MAP is written into the standard XML-format and saved
# at the location specified by outfile.)
proc map_result_to_xml {infile_map outfile_xml} {
	set in_map [open $infile_map]
	set d0 ""
	set d1 ""
	# ignore first three lines
	set line [result_nextline $in_map]
	set line [result_nextline $in_map]
	set line [result_nextline $in_map]
	# get order
	set order [result_nextline $in_map]
	# get rows for matrix D0
	array set D0row_entries {}
	# ignore first line
	set line [result_nextline $in_map]
	for {set i 0} {$i < $order} {incr i} {
		# get columns
		set number_cols ""
		set diag ""
		while {$number_cols == ""} {
			set number_cols [result_nextline $in_map]
			set diag [lindex $number_cols 1]
			set new_entry "\{[list $i $i]\} $diag"
			array set D0row_entries $new_entry
			set number_cols [lindex $number_cols 0]
		}
		for {set j 0} {$j < $number_cols} {incr j} {
			# get entries
			set entry [result_nextline $in_map]
			set pos [lindex $entry 0]
			set value [lindex $entry 1]
			set new_entry "\{[list $i $pos]\} $value"
			array set D0row_entries $new_entry
		}
	}
	# get rows for matrix D1
	array set D1row_entries {}
	# ignore first line
	set line [result_nextline $in_map]
	for {set i 0} {$i < $order} {incr i} {
		# get columns
		set number_cols ""
		set diag ""
		while {$number_cols == ""} {
			set number_cols [result_nextline $in_map]
		}
		for {set j 0} {$j < $number_cols} {incr j} {
			# get entries
			set entry [result_nextline $in_map]
			set pos [lindex $entry 0]
			set value [lindex $entry 1]
			set new_entry "\{[list $i $pos]\} $value"
			array set D1row_entries $new_entry
		}
	}
	# create string from the entries
	for {set i 0} {$i < $order} {incr i} {
		set D0line ""
		set D1line ""
		for {set j 0} {$j < $order} {incr j} {
			if {[array get D0row_entries "[list $i $j]"] != [list]} {
				# file contained entry for element of D0
				set D0line "$D0line [lindex [array get D0row_entries [list $i $j]] 1]"
			} else {
				# non-diagonal element => set to zero
				set D0line "$D0line 0"
			}
			if {[array get D1row_entries "[list $i $j]"] != [list]} {
				# file contained entry for element of D1
				set D1line "$D1line [lindex [array get D1row_entries [list $i $j]] 1]"
			} else {
				# set to zero
				set D1line "$D1line 0"
			}
		}
		if {$d0 == ""} {
			set d0 $D0line
		} else {
			set d0 "$d0 \n$D0line"
		}
		if {$d1 == ""} {
			set d1 $D1line
		} else {
			set d1 "$d1 \n$D1line"
		}
	}
	# write MAP file
	if {$outfile_xml != ""} {
		set out_map [open $outfile_xml w+]
		puts $out_map "<?xml version=\"1.0\"?>"
		puts $out_map "<profido>"
		puts $out_map "<map>"
		puts $out_map "<states>$order</states>"
		puts $out_map "<d0>$d0"
		puts $out_map "</d0>"
		puts $out_map "<d1>$d1"
		puts $out_map "</d1>"
		puts $out_map "</map>"
		puts $out_map "</profido>"
		close $out_map
	}
	close $in_map
}

##################################################################################
##################################################################################
##################################################################################

# This script parses the JMomfit result-file specified by the first parameter
# and calculates the MAP-description from it. Afterwards the MAP is written
# into the standard XML-format and saved at the location specified by the
# second parameter.
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
map_result_to_xml $infile $outfile
exit

