#!/usr/bin/env wish

# This script searches the XML-PH-file specified by infile
# and creates the PH-Input-file used by modgen from it. The 
# created file is saved at the location specified by outfile.
proc xmlph_to_modgen {infile outfile} {
	set inf [open $infile r]
	set content [read $inf]
	close $inf

	# Parse XML-File to a list
	set parsedc [xml2list $content]
	if {[lindex $parsedc 0] != "profido"} {
		puts "Input file is not a ProFiDo XML description!"
		exit
	}
	set found 0
	foreach parsed [lindex $parsedc 2] {
		if {[lindex $parsed 0] == "ph"} {
			set found 1
			set phTag [lindex $parsed 2]
			set d0 ""
			set pi ""
			set states ""
	
			foreach curTag $phTag {		
				if {[lindex $curTag 0] == "states"} {
					set states [lindex [lindex [lindex $curTag 2] 0] 1]
				} else {
					if {[lindex $curTag 0] == "d0"} {
						set d0 [lindex [lindex [lindex $curTag 2] 0] 1]
					} else {
						if {[lindex $curTag 0] == "pi"} {
							set pi [lindex [lindex [lindex $curTag 2] 0] 1]
						}
					}
				}
 	   		}
 	   
			# Check if a valid result was found
			if {($d0 != "") && ($pi != "")} {
				set outf [open $outfile w+]	
		 	   	puts $outf "model = cph" 
 			   	puts $outf "T = $d0"
 	   			puts $outf "tau = $pi"
				close $outf
			} else {
				puts "Error: File $infile does not contain any known information."
			}			
		}
	}
	if {$found == 0} {
		puts "Error: File $infile does not contain PH information."
	}
}

# This methode parses an XML-File into a tcl-list. 
# This code was copied from the TCL-WiKi (http://wiki.tcl.tk/)
 proc xml2list xml {
     regsub -all {>\s*<} [string trim $xml " \n\t<>"] "\} \{" xml
     set xml [string map {> "\} \{#text \{" < "\}\} \{"}  $xml]

     set res ""   ;# string to collect the result
     set stack {} ;# track open tags
     set rest {}
     foreach item "{$xml}" {
	## Ignore XML-comments
	 if {[string range $item 0 0] != "?"} {
		switch -regexp -- $item {
		^# {append res "{[lrange $item 0 end]} " ; #text item}
		^/ {
			regexp {/(.+)} $item -> tagname ;# end tag
			set expected [lindex $stack end]
			if {$tagname!=$expected} {error "$item != $expected"}
			set stack [lrange $stack 0 end-1]
			append res "\}\} "
		}
		/$ { # singleton - start and end in one <> group
		regexp {([^ ]+)( (.+))?/$} $item -> tagname - rest
		set rest [lrange [string map {= " "} $rest] 0 end]
		append res "{$tagname [list $rest] {}} "
		}
		default {
		set tagname [lindex $item 0] ;# start tag
		set rest [lrange [string map {= " "} $item] 1 end]
		lappend stack $tagname
		append res "\{$tagname [list $rest] \{"
		}
		}
		if {[llength $rest]%2} {error "att's not paired: $rest"}
		}
	}


     if [llength $stack] {error "unresolved: $stack"}
     string map {"\} \}" "\}\}"} [lindex $res 0]
 }


##################################################################################
##################################################################################
##################################################################################

# This script parses the XML-PH-file specified by its first parameter
# and creates the PH-Input-file used by modgen from it. The created
# file is saved at the location specified by the second parameter.
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
xmlph_to_modgen $infile $outfile
exit

