#!/usr/bin/env wish

proc xml_to_arta {infile_xml outfile} {
	set in_xml [open $infile_xml]
	set out_arta [open $outfile w+]
	set xml_desc ""
	while {![eof $in_xml]} {
		set xml_desc "$xml_desc [gets $in_xml]"
	}
	regexp {<arta>(.*)</arta>} $xml_desc arta arta_xml_desc
	regexp {<dist>(.*)</dist>} $arta_xml_desc arta dist_xml_desc
	regexp {<arima>(.*)</arima>} $arta_xml_desc arta arima_xml_desc
	regexp {<arcount>(.*)</arcount>} $arima_xml_desc arta arima_arcount
	regexp {<ar>(.*)</ar>} $arima_xml_desc arta arima_ar
	regexp {<ma>(.*)</ma>} $arima_xml_desc arta arima_ma
	regexp {<macount>(.*)</macount>} $arima_xml_desc arta arima_macount
	regexp {<variance>(.*)</variance>} $arima_xml_desc arta arima_variance
	puts $out_arta "model = arta"
	# base process
	puts $out_arta "order = $arima_arcount, $arima_macount"
	if {[string length [string trim $arima_ar]] != 0} {
		puts $out_arta "ar = $arima_ar"
	}
	if {[string length [string trim $arima_ma]] != 0} {
		puts $out_arta "ma = $arima_ma";
	}
	puts $out_arta "ar_mean = 0.0"
	puts $out_arta "ar_variance = $arima_variance"
	# marginal distribution
	if {[regexp {<expodist>(.*)</expodist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<mean>(.*)</mean>} $distc_xml_desc arta param1_xml_desc
		puts $out_arta "distribution = exponential"
		puts $out_arta "rate = [expr 1.0 / $param1_xml_desc]"
	}
	if {[regexp {<normaldist>(.*)</normaldist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<mean>(.*)</mean>} $distc_xml_desc arta param1_xml_desc
		regexp {<std>(.*)</std>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = normal"
		puts $out_arta "mean = $param1_xml_desc"
		puts $out_arta "var = [expr $param2_xml_desc * $param2_xml_desc]"
	}
	if {[regexp {<lognormaldist>(.*)</lognormaldist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<mu>(.*)</mu>} $distc_xml_desc arta param1_xml_desc
		regexp {<sigma>(.*)</sigma>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = lognormal"
		puts $out_arta "mu = $param1_xml_desc"
		puts $out_arta "sigma2 = [expr $param2_xml_desc * $param2_xml_desc]"
	}
	if {[regexp {<johnsondist>(.*)</johnsondist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<type>(.*)</type>} $distc_xml_desc arta param1_xml_desc
		regexp {<shape1>(.*)</shape1>} $distc_xml_desc arta param2_xml_desc
		regexp {<shape2>(.*)</shape2>} $distc_xml_desc arta param3_xml_desc
		regexp {<location>(.*)</location>} $distc_xml_desc arta param4_xml_desc
		regexp {<scale>(.*)</scale>} $distc_xml_desc arta param5_xml_desc
		puts $out_arta "distribution = johnson"
		puts $out_arta "johnson = [string toupper $param1_xml_desc]"
		puts $out_arta "gamma = $param2_xml_desc"
		puts $out_arta "delta = $param3_xml_desc"
		puts $out_arta "xi = $param4_xml_desc"
		puts $out_arta "lambda = $param5_xml_desc"
	}
	if {[regexp {<uniformdist>(.*)</uniformdist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<a>(.*)</a>} $distc_xml_desc arta param1_xml_desc
		regexp {<b>(.*)</b>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = uniform"
		puts $out_arta "a = $param1_xml_desc"
		puts $out_arta "b = $param1_xml_desc"
	}
	if {[regexp {<weibulldist>(.*)</weibulldist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<shape>(.*)</shape>} $distc_xml_desc arta param1_xml_desc
		regexp {<scale>(.*)</scale>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = weibull"
		puts $out_arta "alpha = $param1_xml_desc"
		puts $out_arta "beta = $param2_xml_desc"
	}
	if {[regexp {<triangulardist>(.*)</triangulardist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<a>(.*)</a>} $distc_xml_desc arta param1_xml_desc
		regexp {<b>(.*)</b>} $distc_xml_desc arta param2_xml_desc
		regexp {<c>(.*)</c>} $distc_xml_desc arta param3_xml_desc
		puts $out_arta "distribution = triangular"
		puts $out_arta "a = $param1_xml_desc"
		puts $out_arta "b = $param2_xml_desc"
		puts $out_arta "c = $param3_xml_desc"
	}
	if {[regexp {<erlangdist>(.*)</erlangdist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<phases>(.*)</phases>} $distc_xml_desc arta param1_xml_desc
		regexp {<rate>(.*)</rate>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = erlang"
		puts $out_arta "phases = $param1_xml_desc"
		puts $out_arta "rate = $param2_xml_desc"
	}
	if {[regexp {<gammadist>(.*)</gammadist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<shape>(.*)</shape>} $distc_xml_desc arta param1_xml_desc
		regexp {<scale>(.*)</scale>} $distc_xml_desc arta param2_xml_desc
		puts $out_arta "distribution = gamma"
		puts $out_arta "alpha = $param1_xml_desc"
		puts $out_arta "beta = $param2_xml_desc"
	}
	if {[regexp {<chisquaredist>(.*)</chisquaredist>} $dist_xml_desc arta distc_xml_desc] != 0} {
		regexp {<deg>(.*)</deg>} $distc_xml_desc arta param1_xml_desc
		puts $out_arta "distribution = chisquare"
		puts $out_arta "k = $param1_xml_desc"
	}
	close $in_xml
	close $out_arta
}

##################################################################################
##################################################################################
##################################################################################

# This script converts a XML description of an ARTA process
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
xml_to_arta $infile $outfile
exit

