#!/bin/sh
# -----------------------------
# Export of workflow: batch
# -----------------------------

# This workflow was created with ProFiDo 1.2


# Before any further steps, check if all required external binaries are present in the path.
requirements="sh wish Rscript"
missingCommands=""
for curCmd in $requirements; do
	if ! [ `command -v $curCmd` ]; then
		missingCommands="$missingCommands\n$curCmd"
	fi
done
if ! [ -z "$missingCommands" ]; then
	echo "Warning, the following required external commands were not found on your system.\nPlease ensure their presence before executing the workflow.$missingCommands"
	exit 1
fi

# Before the actual execution, variables are set. To adapt the
# batch to another location just change the BASEDIR variable.
export BASEDIR="/home/gerloff/cvsshk/profido/rsc/export_samples"


# Derived folders: 
export RESULTDIR="${BASEDIR}/Results" 
export SOFTWAREDIR="${BASEDIR}/Software" 


# ----- Convert non-XML files ----- 

"${SOFTWAREDIR}/scripts/TraceToXMLTraceWithValueCount.sh" "${HOME}/ProFiDo/lbl3_dat.trace" "${RESULTDIR}/lbl3_dat.trace.xml"

# ----- Execute Jobs ----- 
# Execute G-FIT
cd "${SOFTWAREDIR}/Gfit/"

./gfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_G-FIT5.xml" -a 5 5 5 -e 1.0e-7
# Execute MAP_EM
cd "${SOFTWAREDIR}/EM/"


./EM.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_G-FIT5.xml" "${RESULTDIR}/MAP5_EM.xml" 0.005 50 5 0.5 123
# Execute Momfit
cd "${SOFTWAREDIR}/momfit/"

./momfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_Momfit7.xml" 5 4 10000 1.0e-7 100000 13
# Execute JMomfit
cd "${SOFTWAREDIR}/jmomfit/"


./jmomfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_Momfit7.xml" "${RESULTDIR}/MAP4_JM.xml" 5
# Execute ARIMA Fitting
cd "${SOFTWAREDIR}/arimafit/"
"${SOFTWAREDIR}/scripts/CreateLinkToRefValueTarget.tcl" "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/lbl3_dat.trace.xml.conv"
./arimafit.r "${RESULTDIR}/lbl3_dat.trace.xml.conv" "${RESULTDIR}/ARMA_6_3.xml" 6 0 3
# Execute Plot
cd "${SOFTWAREDIR}/Plot/"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP5_EM.xml" "${RESULTDIR}/MAP5_EM.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP4_JM.xml" "${RESULTDIR}/MAP4_JM.xml.conv"
"${SOFTWAREDIR}/scripts/CreateLinkToRefValueTarget.tcl" "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/lbl3_dat.trace.xml.conv"
"${SOFTWAREDIR}/scripts/ARIMAToModgen.tcl" "${RESULTDIR}/ARMA_6_3.xml" "${RESULTDIR}/ARMA_6_3.xml.conv"
./plot.exe -format=ps -start=0.0 -end=6.0 -plot=cdf -output="${RESULTDIR}/batch_Plot10.xml.org" -trace "${RESULTDIR}/lbl3_dat.trace.xml.conv" -map "${RESULTDIR}/MAP5_EM.xml.conv" -map "${RESULTDIR}/MAP4_JM.xml.conv" -arima "${RESULTDIR}/ARMA_6_3.xml.conv"
"${SOFTWAREDIR}/scripts/ModgenPSToXMLPS.sh" "${RESULTDIR}/batch_Plot10.xml.org" "${RESULTDIR}/batch_Plot10.xml"
# Execute Plot
cd "${SOFTWAREDIR}/Plot/"
"${SOFTWAREDIR}/scripts/CreateLinkToRefValueTarget.tcl" "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/lbl3_dat.trace.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP5_EM.xml" "${RESULTDIR}/MAP5_EM.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP4_JM.xml" "${RESULTDIR}/MAP4_JM.xml.conv"
"${SOFTWAREDIR}/scripts/ARIMAToModgen.tcl" "${RESULTDIR}/ARMA_6_3.xml" "${RESULTDIR}/ARMA_6_3.xml.conv"
./plot.exe -format=ps -start=1 -end=20 -plot=ac -output="${RESULTDIR}/batch_Plot11.xml.org" -trace "${RESULTDIR}/lbl3_dat.trace.xml.conv" -map "${RESULTDIR}/MAP5_EM.xml.conv" -map "${RESULTDIR}/MAP4_JM.xml.conv" -arima "${RESULTDIR}/ARMA_6_3.xml.conv"
"${SOFTWAREDIR}/scripts/ModgenPSToXMLPS.sh" "${RESULTDIR}/batch_Plot11.xml.org" "${RESULTDIR}/batch_Plot11.xml"


# ----- Convert XML-Files to external outputs ----- 

"${SOFTWAREDIR}/scripts/CreateCopyOffRefValueTarget.tcl" "${RESULTDIR}/batch_Plot10.xml" "${HOME}/ProFiDo/example1_cdf.ps"
"${SOFTWAREDIR}/scripts/CreateCopyOffRefValueTarget.tcl" "${RESULTDIR}/batch_Plot11.xml" "${HOME}/ProFiDo/example1_ac.ps"